/*
 * Copyright (C) 2012-2024 Japan Smartphone Security Association
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jssec.android.sqlite.task;

import org.jssec.android.sqlite.CommonData;
import org.jssec.android.sqlite.DataValidator;
import org.jssec.android.sqlite.MainActivity;
import org.jssec.android.sqlite.R;

import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.os.AsyncTask;
import android.util.Log;


//データ検索タスク
public class DataSearchTask extends AsyncTask<String, Void, Cursor> {
    private MainActivity    mActivity;
    private SQLiteDatabase  mSampleDB;

    public DataSearchTask(SQLiteDatabase db, MainActivity activity) {
        mSampleDB = db;
        mActivity = activity;
    }

    @Override
    protected Cursor doInBackground(String... params) {
        String  idno = params[0];
        String  name = params[1];
        String  info = params[2];
        String  cols[]  =   {"_id", "idno","name","info"};

        Cursor cur;

        //★ポイント3★ SQLインジェクションの保険的な対策としてアプリ要件に従って入力値をチェックする
        if (!DataValidator.validateData(idno, name, info))
            {
                return null;
            }

        //引数が全部nullだったら全件検索する）
        if ((idno == null || idno.length() == 0) &&
            (name == null || name.length() == 0) &&
            (info == null || info.length() == 0) ) {
            try {
                cur = mSampleDB.query(CommonData.TABLE_NAME, cols,
                                null, null, null, null, null);
            } catch (SQLException e) {
                Log.e(DataSearchTask.class.toString(),
                        mActivity.getString(R.string.SEARCHING_ERROR_MESSAGE));
                return null;
            }
            return cur;
        }

        //Noが指定されていたらNoで検索
        if (idno != null && idno.length() > 0) {
            String selectionArgs[] = {idno};

            try {
                //★ポイント2★ SQLインジェクションの対策として入力値をSQL文に使用する場合にはプレースホルダを使用する
                cur =mSampleDB.query(CommonData.TABLE_NAME, cols,
                            "idno = ?", selectionArgs, null, null, null);
            } catch (SQLException e) {
                Log.e(DataSearchTask.class.toString(),
                        mActivity.getString(R.string.SEARCHING_ERROR_MESSAGE));
                return null;
            }
            return cur;
        }

        //Nameが指定されていたらNameで完全一致検索
        if (name != null && name.length() > 0) {
            String selectionArgs[] = {name};
            try {
                //★ポイント2★ SQLインジェクションの対策として入力値をSQL文に使用する場合にはプレースホルダを使用する
                cur = mSampleDB.query(CommonData.TABLE_NAME, cols,
                        "name = ?", selectionArgs, null, null, null);
            } catch (SQLException e) {
                Log.e(DataSearchTask.class.toString(),
                        mActivity.getString(R.string.SEARCHING_ERROR_MESSAGE));
                return null;
            }
            return cur;
        }

        //それ以外の場合はinfoを条件にして部分一致検索
        //入力として受け取ったinfo内の$をエスケープ
        String argString = info.replaceAll("@", "@@");
        //入力として受け取ったinfo内の%をエスケープ
        argString = argString.replaceAll("%", "@%");
        //入力として受け取ったinfo内の_をエスケープ
        argString = argString.replaceAll("_", "@_");
        String selectionArgs[] = {argString};

        try {
            //★ポイント2★ SQLインジェクションの対策として入力値をSQL文に使用する場合にはプレースホルダを使用する
            cur = mSampleDB.query(CommonData.TABLE_NAME, cols,
                        "info LIKE '%' || ? || '%' ESCAPE '@'",
                                selectionArgs, null, null, null);
        } catch (SQLException e) {
            Log.e(DataSearchTask.class.toString(),
                    mActivity.getString(R.string.SEARCHING_ERROR_MESSAGE));
            return null;
        }
        return cur;
    }

    @Override
    protected void onPostExecute(Cursor resultCur) {
        mActivity.updateCursor(resultCur);
    }
}
